'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleWebhook = undefined;

let handlePullRequest = (() => {
  var _ref = _asyncToGenerator(function* (event) {
    if (event.action !== 'opened' && event.action !== 'synchronize') {
      console.log('Unhandled event action: ', event.action);
      return;
    }

    const pull = event.pull_request;
    const otherBranchRemoteRef = `refs/pull/${pull.number}/head`;
    const otherBranchLocalRef = `refs/remotes/origin/PR-${pull.number}`;
    const baseBranchRemoteRef = `refs/heads/${pull.base.ref}`;
    const baseBranchLocalRef = `refs/remotes/origin/${pull.base.ref}`;
    const repo = event.repository.full_name;

    const substitutions = {
      _REPOSITORY_URL: pull.base.repo.clone_url,
      _OTHER_BRANCH_REMOTE_REF: otherBranchRemoteRef,
      _OTHER_BRANCH_LOCAL_REF: otherBranchLocalRef,
      _BASE_BRANCH_REMOTE_REF: baseBranchRemoteRef,
      _BASE_BRANCH_LOCAL_REF: baseBranchLocalRef,
      [_constants.STATUSES_URL_KEY]: pull.statuses_url,
      [_constants.COMMENTS_URL_KEY]: pull.comments_url
    };

    const prTag = `pr.${pull.number}`;
    const tags = [`repo.${event.repository.name.replace('_', '-')}`, `sender.${event.sender.login}`, prTag];

    const existingBuilds = yield _gcloud.googleApis.listCloudbuilds(`tags="${prTag}"`);
    existingBuilds.filter(function (build) {
      return build.status === 'QUEUED' || build.status === 'WORKING';
    }).forEach(function (build) {
      console.log(`Found existing build ${build.id}. Cancelling.`);
      _gcloud.googleApis.cancelCloudbuild(build.id);
    });
    console.log(`Starting cloud build for pull request ${pull.number}.`);
    console.log('Repos', _config2.default.repos);
    console.log('Repo', repo);
    yield _gcloud.googleApis.startCloudbuild(_config2.default.repos[repo].cloudbuild, substitutions, tags);
  });

  return function handlePullRequest(_x) {
    return _ref.apply(this, arguments);
  };
})();

let handleWebhook = exports.handleWebhook = (() => {
  var _ref2 = _asyncToGenerator(function* (req, res) {
    if (!(0, _verify2.default)((yield _keymanager.keyManager.getWebhookSecret()), req.rawBody.toString(), req.get('X-Hub-Signature'))) {
      console.error('Invalid signature.');
      res.status(400).end();
      return;
    }

    const event = req.body;
    const eventType = req.get('X-GitHub-Event');
    switch (eventType) {
      case 'pull_request':
        yield handlePullRequest(event);
        break;
      default:
        console.log('Unhandled event type: ', eventType);
        break;
    }
    res.status(200).end();
  });

  return function handleWebhook(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
})();

var _verify = require('@octokit/webhooks/verify');

var _verify2 = _interopRequireDefault(_verify);

var _keymanager = require('./keymanager');

var _gcloud = require('./gcloud');

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * MIT License
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Copyright (c) 2017 Choko (choko@curioswitch.org)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Permission is hereby granted, free of charge, to any person obtaining a copy
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * of this software and associated documentation files (the "Software"), to deal
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * in the Software without restriction, including without limitation the rights
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * copies of the Software, and to permit persons to whom the Software is
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * furnished to do so, subject to the following conditions:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * The above copyright notice and this permission notice shall be included in all
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * copies or substantial portions of the Software.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * SOFTWARE.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            */